@@ -672,7 +672,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         """
         metadata = {}  # type: Dict[unicode, Any]
         metadata['uid'] = self.config.epub_uid
-        metadata['title'] = self.config.epub_title
+        metadata['title'] = self.esc(self.config.epub_title)
         metadata['level'] = level
         metadata['navpoints'] = navpoints
         return metadata
@@ -707,9 +707,9 @@ class EpubBuilder(StandaloneHTMLBuilder):
         """
         logger.info('writing %s file...', outname)
         epub_filename = path.join(outdir, outname)
-        with ZipFile(epub_filename, 'w', ZIP_DEFLATED) as epub:  # type: ignore
-            epub.write(path.join(outdir, 'mimetype'), 'mimetype', ZIP_STORED)  # type: ignore
+        with ZipFile(epub_filename, 'w', ZIP_DEFLATED) as epub:
+            epub.write(path.join(outdir, 'mimetype'), 'mimetype', ZIP_STORED)
             for filename in [u'META-INF/container.xml', u'content.opf', u'toc.ncx']:
-                epub.write(path.join(outdir, filename), filename, ZIP_DEFLATED)  # type: ignore
+                epub.write(path.join(outdir, filename), filename, ZIP_DEFLATED)
             for filename in self.files:
-                epub.write(path.join(outdir, filename), filename, ZIP_DEFLATED)  # type: ignore
+                epub.write(path.join(outdir, filename), filename, ZIP_DEFLATED)
