@@ -5,7 +5,7 @@
 
     The C language domain.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -15,12 +15,22 @@ import string
 from docutils import nodes
 
 from sphinx import addnodes
-from sphinx.roles import XRefRole
-from sphinx.locale import l_, _
-from sphinx.domains import Domain, ObjType
 from sphinx.directives import ObjectDescription
-from sphinx.util.nodes import make_refnode
+from sphinx.domains import Domain, ObjType
+from sphinx.locale import l_, _
+from sphinx.roles import XRefRole
+from sphinx.util import logging
 from sphinx.util.docfields import Field, TypedField
+from sphinx.util.nodes import make_refnode
+
+if False:
+    # For type annotation
+    from typing import Any, Dict, Iterator, List, Tuple  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+    from sphinx.builders import Builder  # NOQA
+    from sphinx.environment import BuildEnvironment  # NOQA
+
+logger = logging.getLogger(__name__)
 
 
 # RE to split at word boundaries
@@ -74,10 +84,11 @@ class CObject(ObjectDescription):
     ))
 
     def _parse_type(self, node, ctype):
+        # type: (nodes.Node, unicode) -> None
         # add cross-ref nodes for all words
-        for part in [_f for _f in wsplit_re.split(ctype) if _f]:
+        for part in [_f for _f in wsplit_re.split(ctype) if _f]:  # type: ignore
             tnode = nodes.Text(part, part)
-            if part[0] in string.ascii_letters+'_' and \
+            if part[0] in string.ascii_letters + '_' and \
                part not in self.stopwords:
                 pnode = addnodes.pending_xref(
                     '', refdomain='c', reftype='type', reftarget=part,
@@ -88,11 +99,12 @@ class CObject(ObjectDescription):
                 node += tnode
 
     def _parse_arglist(self, arglist):
+        # type: (unicode) -> Iterator[unicode]
         while True:
-            m = c_funcptr_arg_sig_re.match(arglist)
+            m = c_funcptr_arg_sig_re.match(arglist)  # type: ignore
             if m:
                 yield m.group()
-                arglist = c_funcptr_arg_sig_re.sub('', arglist)
+                arglist = c_funcptr_arg_sig_re.sub('', arglist)  # type: ignore
                 if ',' in arglist:
                     _, arglist = arglist.split(',', 1)
                 else:
@@ -106,11 +118,12 @@ class CObject(ObjectDescription):
                     break
 
     def handle_signature(self, sig, signode):
+        # type: (unicode, addnodes.desc_signature) -> unicode
         """Transform a C signature into RST nodes."""
         # first try the function pointer signature regex, it's more specific
-        m = c_funcptr_sig_re.match(sig)
+        m = c_funcptr_sig_re.match(sig)  # type: ignore
         if m is None:
-            m = c_sig_re.match(sig)
+            m = c_sig_re.match(sig)  # type: ignore
         if m is None:
             raise ValueError('no match')
         rettype, name, arglist, const = m.groups()
@@ -151,7 +164,7 @@ class CObject(ObjectDescription):
             arg = arg.strip()
             param = addnodes.desc_parameter('', '', noemph=True)
             try:
-                m = c_funcptr_arg_sig_re.match(arg)
+                m = c_funcptr_arg_sig_re.match(arg)  # type: ignore
                 if m:
                     self._parse_type(param, m.group(1) + '(')
                     param += nodes.emphasis(m.group(2), m.group(2))
@@ -162,7 +175,7 @@ class CObject(ObjectDescription):
                     ctype, argname = arg.rsplit(' ', 1)
                     self._parse_type(param, ctype)
                     # separate by non-breaking space in the output
-                    param += nodes.emphasis(' '+argname, u'\xa0'+argname)
+                    param += nodes.emphasis(' ' + argname, u'\xa0' + argname)
             except ValueError:
                 # no argument name given, only the type
                 self._parse_type(param, arg)
@@ -173,6 +186,7 @@ class CObject(ObjectDescription):
         return fullname
 
     def get_index_text(self, name):
+        # type: (unicode) -> unicode
         if self.objtype == 'function':
             return _('%s (C function)') % name
         elif self.objtype == 'member':
@@ -187,6 +201,7 @@ class CObject(ObjectDescription):
             return ''
 
     def add_target_and_index(self, name, sig, signode):
+        # type: (unicode, unicode, addnodes.desc_signature) -> None
         # for C API items we add a prefix since names are usually not qualified
         # by a module name and so easily clash with e.g. section titles
         targetname = 'c.' + name
@@ -209,6 +224,7 @@ class CObject(ObjectDescription):
                                               targetname, '', None))
 
     def before_content(self):
+        # type: () -> None
         self.typename_set = False
         if self.name == 'c:type':
             if self.names:
@@ -216,12 +232,14 @@ class CObject(ObjectDescription):
                 self.typename_set = True
 
     def after_content(self):
+        # type: () -> None
         if self.typename_set:
             self.env.ref_context.pop('c:type', None)
 
 
 class CXRefRole(XRefRole):
     def process_link(self, env, refnode, has_explicit_title, title, target):
+        # type: (BuildEnvironment, nodes.Node, bool, unicode, unicode) -> Tuple[unicode, unicode]  # NOQA
         if not has_explicit_title:
             target = target.lstrip('~')  # only has a meaning for the title
             # if the first character is a tilde, don't display the module/class
@@ -230,7 +248,7 @@ class CXRefRole(XRefRole):
                 title = title[1:]
                 dot = title.rfind('.')
                 if dot != -1:
-                    title = title[dot+1:]
+                    title = title[dot + 1:]
         return title, target
 
 
@@ -262,21 +280,28 @@ class CDomain(Domain):
     }
     initial_data = {
         'objects': {},  # fullname -> docname, objtype
-    }
+    }  # type: Dict[unicode, Dict[unicode, Tuple[unicode, Any]]]
 
     def clear_doc(self, docname):
+        # type: (unicode) -> None
         for fullname, (fn, _l) in list(self.data['objects'].items()):
             if fn == docname:
                 del self.data['objects'][fullname]
 
     def merge_domaindata(self, docnames, otherdata):
-        # XXX check duplicates
+        # type: (List[unicode], Dict) -> None
         for fullname, (fn, objtype) in otherdata['objects'].items():
             if fn in docnames:
+                if fullname in self.data['objects']:
+                    other, _ = self.data['objects'][fullname]
+                    logger.warning('duplicate C object description of %s, '
+                                   'other instance in %s' %
+                                   (fullname, self.env.doc2path(other)))
                 self.data['objects'][fullname] = (fn, objtype)
 
     def resolve_xref(self, env, fromdocname, builder,
                      typ, target, node, contnode):
+        # type: (BuildEnvironment, unicode, Builder, unicode, unicode, nodes.Node, nodes.Node) -> nodes.Node  # NOQA
         # strip pointer asterisk
         target = target.rstrip(' *')
         # becase TypedField can generate xrefs
@@ -290,6 +315,7 @@ class CDomain(Domain):
 
     def resolve_any_xref(self, env, fromdocname, builder, target,
                          node, contnode):
+        # type: (BuildEnvironment, unicode, Builder, unicode, nodes.Node, nodes.Node) -> List[Tuple[unicode, nodes.Node]]  # NOQA
         # strip pointer asterisk
         target = target.rstrip(' *')
         if target not in self.data['objects']:
@@ -300,9 +326,17 @@ class CDomain(Domain):
                               contnode, target))]
 
     def get_objects(self):
+        # type: () -> Iterator[Tuple[unicode, unicode, unicode, unicode, unicode, int]]
         for refname, (docname, type) in list(self.data['objects'].items()):
             yield (refname, refname, type, docname, 'c.' + refname, 1)
 
 
 def setup(app):
+    # type: (Sphinx) -> Dict[unicode, Any]
     app.add_domain(CDomain)
+
+    return {
+        'version': 'builtin',
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
