@@ -11,7 +11,6 @@
 
 import re
 import unicodedata
-from typing import TYPE_CHECKING
 
 from docutils import nodes
 from docutils.parsers.rst import Directive, directives
@@ -26,7 +25,8 @@ from sphinx.roles import XRefRole
 from sphinx.util import ws_re, logging, docname_join
 from sphinx.util.nodes import clean_astext, make_refnode
 
-if TYPE_CHECKING:
+if False:
+    # For type annotation
     from typing import Any, Callable, Dict, Iterator, List, Tuple, Type, Union  # NOQA
     from sphinx.application import Sphinx  # NOQA
     from sphinx.builders import Builder  # NOQA
@@ -547,7 +547,6 @@ class StandardDomain(Domain):
 
     def merge_domaindata(self, docnames, otherdata):
         # type: (List[unicode], Dict) -> None
-        # XXX duplicates?
         for key, data in otherdata['progoptions'].items():
             if data[0] in docnames:
                 self.data['progoptions'][key] = data
@@ -556,6 +555,11 @@ class StandardDomain(Domain):
                 self.data['objects'][key] = data
         for key, data in otherdata['citations'].items():
             if data[0] in docnames:
+                if key in self.data['citations']:
+                    otherdoc, _, _ = self.data['catations']
+                    logger.warning('duplicate citation %s, other instance in %s' %
+                                   (key, self.env.doc2path(otherdoc)),
+                                   type='ref', subtype='citation')
                 self.data['citations'][key] = data
         for key, data in otherdata['citation_refs'].items():
             citation_refs = self.data['citation_refs'].setdefault(key, [])
@@ -564,6 +568,10 @@ class StandardDomain(Domain):
                     citation_refs.append(docname)
         for key, data in otherdata['labels'].items():
             if data[0] in docnames:
+                if key in self.data['labels']:
+                    otherdoc, _, _ = self.data['labels']
+                    logger.warning('duplicate label %s, other instance in %s' %
+                                   (key, self.env.doc2path(otherdoc)))
                 self.data['labels'][key] = data
         for key, data in otherdata['anonlabels'].items():
             if data[0] in docnames:
@@ -715,15 +723,15 @@ class StandardDomain(Domain):
         if not docname:
             return None
 
-        if env.config.numfig is False:
-            logger.warning('numfig is disabled. :numref: is ignored.', location=node)
-            return contnode
-
         target_node = env.get_doctree(docname).ids.get(labelid)
         figtype = self.get_figtype(target_node)
         if figtype is None:
             return None
 
+        if figtype != 'section' and env.config.numfig is False:
+            logger.warning('numfig is disabled. :numref: is ignored.', location=node)
+            return contnode
+
         try:
             fignumber = self.get_fignumber(env, builder, figtype, docname, target_node)
             if fignumber is None:
