@@ -72,6 +72,10 @@ class MathDomain(Domain):
         # type: (Iterable[unicode], Dict) -> None
         for labelid, (doc, eqno) in otherdata['objects'].items():
             if doc in docnames:
+                if labelid in self.data['objects']:
+                    otherdoc, _ = self.data['objects']
+                    logger.warning('duplicate label of equation %s, other instance in %s' %
+                                   (labelid, self.env.doc2path(otherdoc)))
                 self.data['objects'][labelid] = (doc, eqno)
 
     def resolve_xref(self, env, fromdocname, builder, typ, target, node, contnode):
@@ -85,6 +89,7 @@ class MathDomain(Domain):
                 newnode['target'] = target
                 return newnode
             else:
+                # TODO: perhaps use rather a sphinx-core provided prefix here?
                 node_id = make_id('equation-%s' % target)
                 if env.config.math_numfig and env.config.numfig:
                     if docname in env.toc_fignumbers:
